@@ -1,7 +1,7 @@
 #==============================================================================
 # Contains private configuration procedures for tablelist widgets.
 #
-# Copyright (c) 2000-2007  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2000-2008  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #------------------------------------------------------------------------------
@@ -18,7 +18,7 @@ proc tablelist::extendConfigSpecs {} {
     #
     # Extend some elements of the array configSpecs
     #
-    lappend configSpecs(-activestyle)		underline
+    lappend configSpecs(-activestyle)		frame
     lappend configSpecs(-columns)		{}
     lappend configSpecs(-editendcommand)	{}
     lappend configSpecs(-editstartcommand)	{}
@@ -36,7 +36,7 @@ proc tablelist::extendConfigSpecs {} {
     lappend configSpecs(-resizablecolumns)	1
     lappend configSpecs(-resizecursor)		sb_h_double_arrow
     lappend configSpecs(-selecttype)		row
-    lappend configSpecs(-setfocus)		0
+    lappend configSpecs(-setfocus)		1
     lappend configSpecs(-showarrow)		1
     lappend configSpecs(-showlabels)		1
     lappend configSpecs(-showseparators)	0
@@ -49,6 +49,8 @@ proc tablelist::extendConfigSpecs {} {
     lappend configSpecs(-stripeheight)		1
     lappend configSpecs(-targetcolor)		black
     lappend configSpecs(-titlecolumns)		0
+    lappend configSpecs(-tooltipaddcommand)	{}
+    lappend configSpecs(-tooltipdelcommand)	{}
 
     #
     # Append the default values of the configuration options
@@ -85,19 +87,9 @@ proc tablelist::extendConfigSpecs {} {
 	#
 	# Append theme-specific values to some elements of the array configSpecs
 	#
+	setThemeDefaults
+
 	ttk::label $helpLabel -takefocus 0
-	variable themeDefaults
-	setThemeDefaults		;# pupulates the array themeDefaults
-	foreach opt {-labelbackground -labelforeground -labelfont
-		     -labelborderwidth -labelpady
-		     -arrowcolor -arrowdisabledcolor -arrowstyle} {
-	    lappend configSpecs($opt) $themeDefaults($opt)
-	}
-	foreach opt {-background -foreground -disabledforeground
-		     -stripebackground -selectbackground -selectforeground
-		     -selectborderwidth -font} {
-	    lset configSpecs($opt) 3 $themeDefaults($opt)
-	}
 
 	#
 	# Define the header label layout
@@ -466,7 +458,7 @@ proc tablelist::doConfig {win opt val} {
 			([string compare $opt "-arrowdisabledcolor"] == 0 &&
 			 $data(isDisabled))} {
 			foreach w [info commands $data(hdrTxtFrCanv)*] {
-			    fillArrows $w $val
+			    fillArrows $w $val $data(-arrowstyle)
 			}
 		    }
 		}
@@ -484,9 +476,9 @@ proc tablelist::doConfig {win opt val} {
 		    foreach w [info commands $data(hdrTxtFrCanv)*] {
 			createArrows $w $width $height $relief
 			if {$data(isDisabled)} {
-			    fillArrows $w $data(-arrowdisabledcolor)
+			    fillArrows $w $data(-arrowdisabledcolor) $data($opt)
 			} else {
-			    fillArrows $w $data(-arrowcolor)
+			    fillArrows $w $data(-arrowcolor) $data($opt)
 			}
 		    }
 		    if {[llength $data(arrowColList)] > 0} {
@@ -503,7 +495,7 @@ proc tablelist::doConfig {win opt val} {
 		    # the lists of the column fonts and tag
 		    # names, and redisplay the items
 		    #
-		    set selCells [curcellselectionSubCmd $win]
+		    set selCells [curCellSelection $win]
 		    setupColumns $win $val 1
 		    adjustColumns $win allCols 1
 		    adjustColIndex $win data(anchorCol) 1
@@ -536,6 +528,8 @@ proc tablelist::doConfig {win opt val} {
 		-labelcommand2 -
 		-selectmode -
 		-sortcommand -
+		-tooltipaddcommand -
+		-tooltipdelcommand -
 		-yscrollcommand {
 		    set data($opt) $val
 		}
@@ -669,6 +663,7 @@ proc tablelist::doConfig {win opt val} {
 		    $w configure -spacing1 [expr {$spacing + $pixVal}] \
 				 -spacing3 [expr {$spacing + $pixVal + 1}]
 		    $data(lb) configure $opt $val
+		    redisplayWhenIdle $win
 		    updateColorsWhenIdle $win
 		    adjustSepsWhenIdle $win
 		}
@@ -740,6 +735,7 @@ proc tablelist::doConfig {win opt val} {
 		    $w configure -spacing1 [expr {$pixVal + $selectBd}] \
 				 -spacing3 [expr {$pixVal + $selectBd + 1}]
 		    set data($opt) $val
+		    redisplayWhenIdle $win
 		    updateColorsWhenIdle $win
 		    adjustSepsWhenIdle $win
 		}
@@ -915,7 +911,6 @@ proc tablelist::doConfig {win opt val} {
 #------------------------------------------------------------------------------
 proc tablelist::doCget {win opt} {
     upvar ::tablelist::ns${win}::data data
-
     return $data($opt)
 }
 
@@ -1000,7 +995,8 @@ proc tablelist::doColConfig {col win opt val} {
 	    makeColFontAndTagLists $win
 	}
 
-	-changesnipside {
+	-changesnipside -
+	-wrap {
 	    #
 	    # Save the boolean value specified by val in data($col$opt) and
 	    # make sure the given column will be redisplayed at idle time
@@ -1021,8 +1017,7 @@ proc tablelist::doColConfig {col win opt val} {
 
 	-editwindow {
 	    variable editWin
-	    if {[info exists editWin($val-registered)] ||
-		[info exists editWin($val-creationCmd)]} {
+	    if {[info exists editWin($val-creationCmd)]} {
 		set data($col$opt) $val
 	    } else {
 		return -code error "name \"$val\" is not registered\
@@ -1113,6 +1108,8 @@ proc tablelist::doColConfig {col win opt val} {
 	    #
 	    set data(fmtCmdFlagList) \
 		[lreplace $data(fmtCmdFlagList) $col $col $fmtCmdFlag]
+	    set data(hasFmtCmds) \
+		[expr {[lsearch -exact $data(fmtCmdFlagList) 1] >= 0}]
 
 	    #
 	    # Adjust the columns and make sure the specified
@@ -1131,9 +1128,9 @@ proc tablelist::doColConfig {col win opt val} {
 	    set newVal [expr {$val ? 1 : 0}]
 	    if {$newVal != $oldVal} {
 		if {!$canElide} {
-		    set selCells [curcellselectionSubCmd $win]
+		    set selCells [curCellSelection $win]
 		} elseif {$newVal} {
-		    cellselectionSubCmd $win clear 0 $col $data(lastRow) $col
+		    cellSelection $win clear 0 $col $data(lastRow) $col
 		}
 		set data($col$opt) $newVal
 		if {$newVal} {				;# hiding the column
@@ -1141,7 +1138,7 @@ proc tablelist::doColConfig {col win opt val} {
 		    adjustColIndex $win data(anchorCol) 1
 		    adjustColIndex $win data(activeCol) 1
 		    if {$col == $data(editCol)} {
-			canceleditingSubCmd $win
+			doCancelEditing $win
 		    }
 		} else {
 		    incr data(hiddenColCount) -1
@@ -1155,8 +1152,8 @@ proc tablelist::doColConfig {col win opt val} {
 		}
 		if {!$newVal &&
 		    [string compare $data(-selecttype) "row"] == 0} {
-		    foreach row [curselectionSubCmd $win] {
-			selectionSubCmd $win set $row $row
+		    foreach row [curSelection $win] {
+			rowSelection $win set $row $row
 		    }
 		}
 	    }
@@ -1622,8 +1619,8 @@ proc tablelist::doColConfig {col win opt val} {
 	    set idx [expr {3*$col}]
 	    if {$val != [lindex $data(-columns) $idx]} {
 		setupColumns $win [lreplace $data(-columns) $idx $idx $val] 0
+		redisplayColWhenIdle $win $col	;# here before adjustColumns!
 		adjustColumns $win $col 1
-		redisplayColWhenIdle $win $col
 	    }
 	}
     }
@@ -1709,7 +1706,7 @@ proc tablelist::doRowConfig {row win opt val} {
 	    if {[string compare $val ""] == 0} {
 		if {[info exists data($name)]} {
 		    unset data($name)
-		    incr data(tagRefCount) -1
+		    incr data(rowTagRefCount) -1
 		}
 	    } else {
 		#
@@ -1720,25 +1717,31 @@ proc tablelist::doRowConfig {row win opt val} {
 		$w tag configure $tag $opt $val
 		$w tag lower $tag active
 		set line [expr {$row + 1}]
-		set textIdx1 [expr {double($line)}]
-		for {set col 0} {$col < $data(colCount)} {incr col} {
-		    if {$data($col-hide) && !$canElide} {
-			continue
-		    }
+		if {[llength [$w tag nextrange select $line.0 $line.end]]
+		    == 0} {
+		    $w tag add $tag $line.0 $line.end
+		} else {
+		    set textIdx1 [expr {double($line)}]
+		    for {set col 0} {$col < $data(colCount)} {incr col} {
+			if {$data($col-hide) && !$canElide} {
+			    continue
+			}
 
-		    set textIdx2 \
-			[$w search $elide "\t" $textIdx1+1c $line.end]+1c
-		    if {[lsearch -exact [$w tag names $textIdx1] select] < 0} {
-			$w tag add $tag $textIdx1 $textIdx2
+			set textIdx2 \
+			    [$w search $elide "\t" $textIdx1+1c $line.end]+1c
+			if {[lsearch -exact [$w tag names $textIdx1] select]
+			    < 0} {
+			    $w tag add $tag $textIdx1 $textIdx2
+			}
+			set textIdx1 $textIdx2
 		    }
-		    set textIdx1 $textIdx2
 		}
 
 		#
 		# Save val in data($name)
 		#
 		if {![info exists data($name)]} {
-		    incr data(tagRefCount)
+		    incr data(rowTagRefCount)
 		}
 		set data($name) $val
 	    }
@@ -1770,7 +1773,7 @@ proc tablelist::doRowConfig {row win opt val} {
 	    if {[string compare $val ""] == 0} {
 		if {[info exists data($name)]} {
 		    unset data($name)
-		    incr data(tagRefCount) -1
+		    incr data(rowTagRefCount) -1
 		}
 	    } else {
 		#
@@ -1787,23 +1790,21 @@ proc tablelist::doRowConfig {row win opt val} {
 		# Save val in data($name)
 		#
 		if {![info exists data($name)]} {
-		    incr data(tagRefCount)
+		    incr data(rowTagRefCount)
 		}
 		set data($name) $val
 	    }
 
-	    if {[lsearch -exact $data(fmtCmdFlagList) 1] >= 0} {
-		set formattedItem \
-		    [formatItem $win [lrange $item 0 $data(lastCol)]]
-	    } else {
-		set formattedItem [lrange $item 0 $data(lastCol)]
+	    set displayedItem [lrange $item 0 $data(lastCol)]
+	    if {$data(hasFmtCmds)} {
+		set displayedItem [formatItem $win $key $row $displayedItem]
 	    }
 	    set colWidthsChanged 0
 	    set colIdxList {}
 	    set line [expr {$row + 1}]
 	    set textIdx1 $line.1
 	    set col 0
-	    foreach text [strToDispStr $formattedItem] \
+	    foreach text [strToDispStr $displayedItem] \
 		    {pixels alignment} $data(colList) {
 		if {$data($col-hide) && !$canElide} {
 		    incr col
@@ -1813,33 +1814,43 @@ proc tablelist::doRowConfig {row win opt val} {
 		#
 		# Adjust the cell text and the image or window width
 		#
-		if {[string match "*\n*" $text]} {
-		    set multiline 1
-		    set list [split $text "\n"]
-		} else {
-		    set multiline 0
-		}
-		set aux [getAuxData $win $key $col auxType auxWidth]
-		set textSav $text
-		set auxWidthSav $auxWidth
+		set multiline [string match "*\n*" $text]
 		set cellFont [getCellFont $win $key $col]
 		set workPixels $pixels
 		if {$pixels == 0} {		;# convention: dynamic width
+		    set textSav $text
+		    getAuxData $win $key $col auxType auxWidthSav
+
 		    if {$data($col-maxPixels) > 0} {
 			if {$data($col-reqPixels) > $data($col-maxPixels)} {
 			    set workPixels $data($col-maxPixels)
 			}
 		    }
 		}
+		set aux [getAuxData $win $key $col auxType auxWidth $workPixels]
+		set maxTextWidth $workPixels
 		if {$workPixels != 0} {
 		    incr workPixels $data($col-delta)
+		    set maxTextWidth [getMaxTextWidth $workPixels $auxWidth]
+
+		    if {$data($col-wrap) && !$multiline} {
+			if {[font measure $cellFont -displayof $win $text] >
+			    $maxTextWidth} {
+			    set multiline 1
+			}
+		    }
 		}
 		set snipSide $snipSides($alignment,$data($col-changesnipside))
 		if {$multiline} {
+		    set list [split $text "\n"]
+		    if {$data($col-wrap)} {
+			set snipSide ""
+		    }
 		    adjustMlElem $win list auxWidth $cellFont $workPixels \
 				 $snipSide $data(-snipstring)
 		    set msgScript [list ::tablelist::displayText $win $key \
-				   $col [join $list "\n"] $cellFont $alignment]
+				   $col [join $list "\n"] $cellFont \
+				   $maxTextWidth $alignment]
 		} else {
 		    adjustElem $win text auxWidth $cellFont $workPixels \
 			       $snipSide $data(-snipstring)
@@ -1920,7 +1931,7 @@ proc tablelist::doRowConfig {row win opt val} {
 
 	    if {$val} {					;# hiding the row
 		if {![info exists data($name)]} {
-		    selectionSubCmd $win clear $row $row
+		    rowSelection $win clear $row $row
 		    set data($name) 1
 		    incr data(hiddenRowCount)
 		    $w tag add hiddenRow $line.0 $line.end+1c
@@ -1928,7 +1939,7 @@ proc tablelist::doRowConfig {row win opt val} {
 		    adjustRowIndex $win data(anchorRow) 1
 		    adjustRowIndex $win data(activeRow) 1
 		    if {$row == $data(editRow)} {
-			canceleditingSubCmd $win
+			doCancelEditing $win
 		    }
 		}
 	    } else {					;# unhiding the row
@@ -1958,14 +1969,12 @@ proc tablelist::doRowConfig {row win opt val} {
 		#
 		set colWidthsChanged 0
 		set colIdxList {}
-		if {[lsearch -exact $data(fmtCmdFlagList) 1] >= 0} {
-		    set formattedItem \
-			[formatItem $win [lrange $item 0 $data(lastCol)]]
-		} else {
-		    set formattedItem [lrange $item 0 $data(lastCol)]
+		set displayedItem [lrange $item 0 $data(lastCol)]
+		if {$data(hasFmtCmds)} {
+		    set displayedItem [formatItem $win $key $row $displayedItem]
 		}
 		set col 0
-		foreach text [strToDispStr $formattedItem] \
+		foreach text [strToDispStr $displayedItem] \
 			{pixels alignment} $data(colList) {
 		    if {($data($col-hide) && !$canElide) || $pixels != 0} {
 			incr col
@@ -2038,7 +2047,7 @@ proc tablelist::doRowConfig {row win opt val} {
 		# Set data($key$opt) to 0 and deselect the row
 		#
 		set data($key$opt) 0
-		selectionSubCmd $win clear $row $row
+		rowSelection $win clear $row $row
 	    }
 	}
 
@@ -2069,18 +2078,11 @@ proc tablelist::doRowConfig {row win opt val} {
 		$w tag configure $tag -$optTail $val
 		$w tag lower $tag active
 		set line [expr {$row + 1}]
-		set textIdx1 [expr {double($line)}]
-		for {set col 0} {$col < $data(colCount)} {incr col} {
-		    if {$data($col-hide) && !$canElide} {
-			continue
-		    }
-
-		    set textIdx2 \
-			[$w search $elide "\t" $textIdx1+1c $line.end]+1c
-		    if {[lsearch -exact [$w tag names $textIdx1] select] >= 0} {
-			$w tag add $tag $textIdx1 $textIdx2
-		    }
-		    set textIdx1 $textIdx2
+		set selRange [$w tag nextrange select $line.0 $line.end]
+		while {[llength $selRange] != 0} {
+		    foreach {selStart selEnd} $selRange {}
+		    $w tag add $tag $selStart $selEnd
+		    set selRange [$w tag nextrange select $selEnd $line.end]
 		}
 
 		#
@@ -2104,15 +2106,15 @@ proc tablelist::doRowConfig {row win opt val} {
 	    set oldItem [lindex $data(itemList) $row]
 	    set key [lindex $oldItem end]
 	    set newItem [adjustItem $val $data(colCount)]
-	    if {[lsearch -exact $data(fmtCmdFlagList) 1] >= 0} {
-		set formattedItem [formatItem $win $newItem]
+	    if {$data(hasFmtCmds)} {
+		set displayedItem [formatItem $win $key $row $newItem]
 	    } else {
-		set formattedItem $newItem
+		set displayedItem $newItem
 	    }
 	    set line [expr {$row + 1}]
 	    set textIdx1 $line.1
 	    set col 0
-	    foreach text [strToDispStr $formattedItem] \
+	    foreach text [strToDispStr $displayedItem] \
 		    {pixels alignment} $data(colList) {
 		if {$data($col-hide) && !$canElide} {
 		    incr col
@@ -2122,33 +2124,43 @@ proc tablelist::doRowConfig {row win opt val} {
 		#
 		# Adjust the cell text and the image or window width
 		#
-		if {[string match "*\n*" $text]} {
-		    set multiline 1
-		    set list [split $text "\n"]
-		} else {
-		    set multiline 0
-		}
-		set aux [getAuxData $win $key $col auxType auxWidth]
-		set textSav $text
-		set auxWidthSav $auxWidth
+		set multiline [string match "*\n*" $text]
 		set cellFont [getCellFont $win $key $col]
 		set workPixels $pixels
 		if {$pixels == 0} {		;# convention: dynamic width
+		    set textSav $text
+		    getAuxData $win $key $col auxType auxWidthSav
+
 		    if {$data($col-maxPixels) > 0} {
 			if {$data($col-reqPixels) > $data($col-maxPixels)} {
 			    set workPixels $data($col-maxPixels)
 			}
 		    }
 		}
+		set aux [getAuxData $win $key $col auxType auxWidth $workPixels]
+		set maxTextWidth $workPixels
 		if {$workPixels != 0} {
 		    incr workPixels $data($col-delta)
+		    set maxTextWidth [getMaxTextWidth $workPixels $auxWidth]
+
+		    if {$data($col-wrap) && !$multiline} {
+			if {[font measure $cellFont -displayof $win $text] >
+			    $maxTextWidth} {
+			    set multiline 1
+			}
+		    }
 		}
 		set snipSide $snipSides($alignment,$data($col-changesnipside))
 		if {$multiline} {
+		    set list [split $text "\n"]
+		    if {$data($col-wrap)} {
+			set snipSide ""
+		    }
 		    adjustMlElem $win list auxWidth $cellFont $workPixels \
 				 $snipSide $data(-snipstring)
 		    set msgScript [list ::tablelist::displayText $win $key \
-				   $col [join $list "\n"] $cellFont $alignment]
+				   $col [join $list "\n"] $cellFont \
+				   $maxTextWidth $alignment]
 		} else {
 		    adjustElem $win text auxWidth $cellFont $workPixels \
 			       $snipSide $data(-snipstring)
@@ -2185,9 +2197,9 @@ proc tablelist::doRowConfig {row win opt val} {
 			}
 		    } else {
 			set oldText [lindex $oldItem $col]
-			if {[info exists data($col-formatcommand)]} {
-			    set oldText [uplevel #0 $data($col-formatcommand) \
-					 [list $oldText]]
+			if {[lindex $data(fmtCmdFlagList) $col]} {
+			    set oldText \
+				[formatElem $win $key $row $col $oldText]
 			}
 			set oldText [strToDispStr $oldText]
 			set oldElemWidth \
@@ -2317,7 +2329,7 @@ proc tablelist::doCellConfig {row col win opt val} {
 	    if {[string compare $val ""] == 0} {
 		if {[info exists data($name)]} {
 		    unset data($name)
-		    incr data(tagRefCount) -1
+		    incr data(cellTagRefCount) -1
 		}
 	    } else {
 		#
@@ -2341,7 +2353,7 @@ proc tablelist::doCellConfig {row col win opt val} {
 		# Save val in data($name)
 		#
 		if {![info exists data($name)]} {
-		    incr data(tagRefCount)
+		    incr data(cellTagRefCount)
 		}
 		set data($name) $val
 	    }
@@ -2361,8 +2373,7 @@ proc tablelist::doCellConfig {row col win opt val} {
 
 	-editwindow {
 	    variable editWin
-	    if {[info exists editWin($val-registered)] ||
-		[info exists editWin($val-creationCmd)]} {
+	    if {[info exists editWin($val-creationCmd)]} {
 		set key [lindex [lindex $data(itemList) $row] end]
 		set data($key,$col$opt) $val
 	    } else {
@@ -2392,7 +2403,7 @@ proc tablelist::doCellConfig {row col win opt val} {
 	    if {[string compare $val ""] == 0} {
 		if {[info exists data($name)]} {
 		    unset data($name)
-		    incr data(tagRefCount) -1
+		    incr data(cellTagRefCount) -1
 		}
 	    } else {
 		#
@@ -2414,7 +2425,7 @@ proc tablelist::doCellConfig {row col win opt val} {
 		# Save val in data($name)
 		#
 		if {![info exists data($name)]} {
-		    incr data(tagRefCount)
+		    incr data(cellTagRefCount)
 		}
 		set data($name) $val
 	    }
@@ -2423,39 +2434,49 @@ proc tablelist::doCellConfig {row col win opt val} {
 	    # Adjust the cell text and the image or window width
 	    #
 	    set text [lindex $item $col]
-	    if {[info exists data($col-formatcommand)]} {
-		set text [uplevel #0 $data($col-formatcommand) [list $text]]
+	    if {[lindex $data(fmtCmdFlagList) $col]} {
+		set text [formatElem $win $key $row $col $text]
 	    }
 	    set text [strToDispStr $text]
-	    if {[string match "*\n*" $text]} {
-		set multiline 1
-		set list [split $text "\n"]
-	    } else {
-		set multiline 0
-	    }
-	    set aux [getAuxData $win $key $col auxType auxWidth]
-	    set textSav $text
-	    set auxWidthSav $auxWidth
+	    set multiline [string match "*\n*" $text]
 	    set cellFont [getCellFont $win $key $col]
 	    set pixels [lindex $data(colList) [expr {2*$col}]]
 	    set workPixels $pixels
 	    if {$pixels == 0} {			;# convention: dynamic width
+		set textSav $text
+		getAuxData $win $key $col auxType auxWidthSav
+
 		if {$data($col-maxPixels) > 0} {
 		    if {$data($col-reqPixels) > $data($col-maxPixels)} {
 			set workPixels $data($col-maxPixels)
 		    }
 		}
 	    }
+	    set aux [getAuxData $win $key $col auxType auxWidth $workPixels]
+	    set maxTextWidth $workPixels
 	    if {$workPixels != 0} {
 		incr workPixels $data($col-delta)
+		set maxTextWidth [getMaxTextWidth $workPixels $auxWidth]
+
+		if {$data($col-wrap) && !$multiline} {
+		    if {[font measure $cellFont -displayof $win $text] >
+			$maxTextWidth} {
+			set multiline 1
+		    }
+		}
 	    }
 	    set alignment [lindex $data(colList) [expr {2*$col + 1}]]
 	    set snipSide $snipSides($alignment,$data($col-changesnipside))
 	    if {$multiline} {
+		set list [split $text "\n"]
+		if {$data($col-wrap)} {
+		    set snipSide ""
+		}
 		adjustMlElem $win list auxWidth $cellFont $workPixels \
 			     $snipSide $data(-snipstring)
 		set msgScript [list ::tablelist::displayText $win $key \
-			       $col [join $list "\n"] $cellFont $alignment]
+			       $col [join $list "\n"] $cellFont \
+			       $maxTextWidth $alignment]
 	    } else {
 		adjustElem $win text auxWidth $cellFont $workPixels \
 			   $snipSide $data(-snipstring)
@@ -2557,41 +2578,51 @@ proc tablelist::doCellConfig {row col win opt val} {
 	    #
 	    # Adjust the cell text and the image or window width
 	    #
+	    set pixels [lindex $data(colList) [expr {2*$col}]]
+	    set workPixels $pixels
 	    set text [lindex $item $col]
-	    if {[info exists data($col-formatcommand)]} {
-		set text [uplevel #0 $data($col-formatcommand) [list $text]]
+	    if {[lindex $data(fmtCmdFlagList) $col]} {
+		set text [formatElem $win $key $row $col $text]
 	    }
 	    set text [strToDispStr $text]
 	    set oldText $text
-	    if {[string match "*\n*" $text]} {
-		set multiline 1
-		set list [split $text "\n"]
-	    } else {
-		set multiline 0
-	    }
-	    set aux [getAuxData $win $key $col auxType auxWidth]
-	    set textSav $text
-	    set auxWidthSav $auxWidth
+	    set multiline [string match "*\n*" $text]
 	    set cellFont [getCellFont $win $key $col]
-	    set pixels [lindex $data(colList) [expr {2*$col}]]
-	    set workPixels $pixels
 	    if {$pixels == 0} {			;# convention: dynamic width
+		set textSav $text
+		getAuxData $win $key $col auxType auxWidthSav
+
 		if {$data($col-maxPixels) > 0} {
 		    if {$data($col-reqPixels) > $data($col-maxPixels)} {
 			set workPixels $data($col-maxPixels)
 		    }
 		}
 	    }
+	    set aux [getAuxData $win $key $col auxType auxWidth $workPixels]
+	    set maxTextWidth $workPixels
 	    if {$workPixels != 0} {
 		incr workPixels $data($col-delta)
+		set maxTextWidth [getMaxTextWidth $workPixels $auxWidth]
+
+		if {$data($col-wrap) && !$multiline} {
+		    if {[font measure $cellFont -displayof $win $text] >
+			$maxTextWidth} {
+			set multiline 1
+		    }
+		}
 	    }
 	    set alignment [lindex $data(colList) [expr {2*$col + 1}]]
 	    set snipSide $snipSides($alignment,$data($col-changesnipside))
 	    if {$multiline} {
+		set list [split $text "\n"]
+		if {$data($col-wrap)} {
+		    set snipSide ""
+		}
 		adjustMlElem $win list auxWidth $cellFont $workPixels \
 			     $snipSide $data(-snipstring)
 		set msgScript [list ::tablelist::displayText $win $key \
-			       $col [join $list "\n"] $cellFont $alignment]
+			       $col [join $list "\n"] $cellFont \
+			       $maxTextWidth $alignment]
 	    } else {
 		adjustElem $win text auxWidth $cellFont $workPixels \
 			   $snipSide $data(-snipstring)
@@ -2705,6 +2736,89 @@ proc tablelist::doCellConfig {row col win opt val} {
 	    }
 	}
 
+	-stretchwindow {
+	    #
+	    # Save the boolean value specified by val in data($key,$col$opt)
+	    #
+	    set item [lindex $data(itemList) $row]
+	    set key [lindex $item end]
+	    set name $key,$col$opt
+	    if {$val} {
+		set data($name) 1
+	    } elseif {[info exists data($name)]} {
+		unset data($name)
+	    }
+
+	    if {($data($col-hide) && !$canElide) ||
+		($row == $data(editRow) && $col == $data(editCol))} {
+		return ""
+	    }
+
+	    set pixels [lindex $data(colList) [expr {2*$col}]]
+	    set text [lindex $item $col]
+	    if {[lindex $data(fmtCmdFlagList) $col]} {
+		set text [formatElem $win $key $row $col $text]
+	    }
+	    set text [strToDispStr $text]
+	    set multiline [string match "*\n*" $text]
+	    set cellFont [getCellFont $win $key $col]
+	    if {$pixels == 0} {			;# convention: dynamic width
+		if {$data($col-maxPixels) > 0} {
+		    if {$data($col-reqPixels) > $data($col-maxPixels)} {
+			set pixels $data($col-maxPixels)
+		    }
+		}
+	    }
+	    set aux [getAuxData $win $key $col auxType auxWidth $pixels]
+	    set maxTextWidth $pixels
+	    if {$pixels != 0} {
+		incr pixels $data($col-delta)
+		set maxTextWidth [getMaxTextWidth $pixels $auxWidth]
+
+		if {$data($col-wrap) && !$multiline} {
+		    if {[font measure $cellFont -displayof $win $text] >
+			$maxTextWidth} {
+			set multiline 1
+		    }
+		}
+	    }
+	    if {$auxType < 2} {			;# no window
+		return ""
+	    }
+
+	    #
+	    # Adjust the cell text and the window width
+	    #
+	    set alignment [lindex $data(colList) [expr {2*$col + 1}]]
+	    set snipSide $snipSides($alignment,$data($col-changesnipside))
+	    if {$multiline} {
+		set list [split $text "\n"]
+		if {$data($col-wrap)} {
+		    set snipSide ""
+		}
+		adjustMlElem $win list auxWidth $cellFont $pixels \
+			     $snipSide $data(-snipstring)
+		set msgScript [list ::tablelist::displayText $win $key $col \
+			       [join $list "\n"] $cellFont \
+			       $maxTextWidth $alignment]
+	    } else {
+		adjustElem $win text auxWidth $cellFont $pixels \
+			   $snipSide $data(-snipstring)
+	    }
+
+	    #
+	    # Update the text widget's contents between the two tabs
+	    #
+	    findTabs $win [expr {$row + 1}] $col $col tabIdx1 tabIdx2
+	    if {$multiline} {
+		updateMlCell $w $tabIdx1+1c $tabIdx2 $msgScript \
+			     $aux $auxType $auxWidth $alignment
+	    } else {
+		updateCell $w $tabIdx1+1c $tabIdx2 $text \
+			   $aux $auxType $auxWidth $alignment
+	    }
+	}
+
 	-text {
 	    if {$data(isDisabled)} {
 		return ""
@@ -2712,45 +2826,55 @@ proc tablelist::doCellConfig {row col win opt val} {
 
 	    set pixels [lindex $data(colList) [expr {2*$col}]]
 	    set workPixels $pixels
+	    set text $val
+	    set oldItem [lindex $data(itemList) $row]
+	    set key [lindex $oldItem end]
+	    set fmtCmdFlag [lindex $data(fmtCmdFlagList) $col]
+	    if {$fmtCmdFlag} {
+		set text [formatElem $win $key $row $col $text]
+	    }
+	    set text [strToDispStr $text]
+	    set textSav $text
+	    set multiline [string match "*\n*" $text]
+	    set cellFont [getCellFont $win $key $col]
 	    if {$pixels == 0} {			;# convention: dynamic width
+		getAuxData $win $key $col auxType auxWidthSav
+
 		if {$data($col-maxPixels) > 0} {
 		    if {$data($col-reqPixels) > $data($col-maxPixels)} {
 			set workPixels $data($col-maxPixels)
 		    }
 		}
 	    }
+	    set aux [getAuxData $win $key $col auxType auxWidth $workPixels]
+	    set maxTextWidth $workPixels
 	    if {$workPixels != 0} {
 		incr workPixels $data($col-delta)
+		set maxTextWidth [getMaxTextWidth $workPixels $auxWidth]
+
+		if {$data($col-wrap) && !$multiline} {
+		    if {[font measure $cellFont -displayof $win $text] >
+			$maxTextWidth} {
+			set multiline 1
+		    }
+		}
 	    }
 	    set alignment [lindex $data(colList) [expr {2*$col + 1}]]
 
 	    #
 	    # Adjust the cell text and the image or window width
 	    #
-	    set text $val
-	    set fmtCmdFlag [info exists data($col-formatcommand)]
-	    if {$fmtCmdFlag} {
-		set text [uplevel #0 $data($col-formatcommand) [list $text]]
-	    }
-	    set text [strToDispStr $text]
-	    set textSav $text
-	    if {[string match "*\n*" $text]} {
-		set multiline 1
-		set list [split $text "\n"]
-	    } else {
-		set multiline 0
-	    }
-	    set oldItem [lindex $data(itemList) $row]
-	    set key [lindex $oldItem end]
-	    set aux [getAuxData $win $key $col auxType auxWidth]
-	    set auxWidthSav $auxWidth
-	    set cellFont [getCellFont $win $key $col]
 	    set snipSide $snipSides($alignment,$data($col-changesnipside))
 	    if {$multiline} {
+		set list [split $text "\n"]
+		if {$data($col-wrap)} {
+		    set snipSide ""
+		}
 		adjustMlElem $win list auxWidth $cellFont $workPixels \
 			     $snipSide $data(-snipstring)
 		set msgScript [list ::tablelist::displayText $win $key \
-			       $col [join $list "\n"] $cellFont $alignment]
+			       $col [join $list "\n"] $cellFont \
+			       $maxTextWidth $alignment]
 	    } else {
 		adjustElem $win text auxWidth $cellFont $workPixels \
 			   $snipSide $data(-snipstring)
@@ -2805,8 +2929,7 @@ proc tablelist::doCellConfig {row col win opt val} {
 		} else {
 		    set oldText [lindex $oldItem $col]
 		    if {$fmtCmdFlag} {
-			set oldText [uplevel #0 $data($col-formatcommand) \
-				     [list $oldText]]
+			set oldText [formatElem $win $key $row $col $oldText]
 		    }
 		    set oldText [strToDispStr $oldText]
 		    set oldElemWidth \
@@ -2883,6 +3006,8 @@ proc tablelist::doCellConfig {row col win opt val} {
 				   -container 0 -highlightthickness 0 \
 				    -relief flat -takefocus 0
 		    catch {$aux configure -padx 0 -pady 0}
+		    bindtags $aux [lreplace [bindtags $aux] 1 1 \
+				   $data(bodyTag) TablelistBody]
 		    uplevel #0 $val [list $win $row $col $aux.w]
 		}
 		set data($name) $val
@@ -2908,41 +3033,51 @@ proc tablelist::doCellConfig {row col win opt val} {
 	    #
 	    # Adjust the cell text and the image or window width
 	    #
+	    set pixels [lindex $data(colList) [expr {2*$col}]]
+	    set workPixels $pixels
 	    set text [lindex $item $col]
-	    if {[info exists data($col-formatcommand)]} {
-		set text [uplevel #0 $data($col-formatcommand) [list $text]]
+	    if {[lindex $data(fmtCmdFlagList) $col]} {
+		set text [formatElem $win $key $row $col $text]
 	    }
 	    set text [strToDispStr $text]
 	    set oldText $text
-	    if {[string match "*\n*" $text]} {
-		set multiline 1
-		set list [split $text "\n"]
-	    } else {
-		set multiline 0
-	    }
-	    set aux [getAuxData $win $key $col auxType auxWidth]
-	    set textSav $text
-	    set auxWidthSav $auxWidth
+	    set multiline [string match "*\n*" $text]
 	    set cellFont [getCellFont $win $key $col]
-	    set pixels [lindex $data(colList) [expr {2*$col}]]
-	    set workPixels $pixels
 	    if {$pixels == 0} {			;# convention: dynamic width
+		set textSav $text
+		getAuxData $win $key $col auxType auxWidthSav
+
 		if {$data($col-maxPixels) > 0} {
 		    if {$data($col-reqPixels) > $data($col-maxPixels)} {
 			set workPixels $data($col-maxPixels)
 		    }
 		}
 	    }
+	    set aux [getAuxData $win $key $col auxType auxWidth $workPixels]
+	    set maxTextWidth $workPixels
 	    if {$workPixels != 0} {
 		incr workPixels $data($col-delta)
+		set maxTextWidth [getMaxTextWidth $workPixels $auxWidth]
+
+		if {$data($col-wrap) && !$multiline} {
+		    if {[font measure $cellFont -displayof $win $text] >
+			$maxTextWidth} {
+			set multiline 1
+		    }
+		}
 	    }
 	    set alignment [lindex $data(colList) [expr {2*$col + 1}]]
 	    set snipSide $snipSides($alignment,$data($col-changesnipside))
 	    if {$multiline} {
+		set list [split $text "\n"]
+		if {$data($col-wrap)} {
+		    set snipSide ""
+		}
 		adjustMlElem $win list auxWidth $cellFont $workPixels \
 			     $snipSide $data(-snipstring)
 		set msgScript [list ::tablelist::displayText $win $key \
-			       $col [join $list "\n"] $cellFont $alignment]
+			       $col [join $list "\n"] $cellFont \
+			       $maxTextWidth $alignment]
 	    } else {
 		adjustElem $win text auxWidth $cellFont $workPixels \
 			   $snipSide $data(-snipstring)
@@ -2955,7 +3090,7 @@ proc tablelist::doCellConfig {row col win opt val} {
 		# and insert the text and the auxiliary object
 		#
 		findTabs $win [expr {$row + 1}] $col $col tabIdx1 tabIdx2
-		if {$auxType != 2 || $keepAux} {
+		if {$auxType < 2 || $keepAux} {
 		    if {$multiline} {
 			updateMlCell $w $tabIdx1+1c $tabIdx2 $msgScript \
 				     $aux $auxType $auxWidth $alignment
@@ -3047,6 +3182,15 @@ proc tablelist::doCellCget {row col win opt} {
 	    return [getEditWindow $win $row $col]
 	}
 
+	-stretchwindow {
+	    set key [lindex [lindex $data(itemList) $row] end]
+	    if {[info exists data($key,$col$opt)]} {
+		return $data($key,$col$opt)
+	    } else {
+		return 0
+	    }
+	}
+
 	-text {
 	    return [lindex [lindex $data(itemList) $row] $col]
 	}
@@ -3070,7 +3214,6 @@ proc tablelist::doCellCget {row col win opt} {
 #------------------------------------------------------------------------------
 proc tablelist::makeListVar {win varName} {
     upvar ::tablelist::ns${win}::data data
-
     if {[string compare $varName ""] == 0} {
 	#
 	# If there is an old list variable associated with the
@@ -3135,7 +3278,6 @@ proc tablelist::makeListVar {win varName} {
 #------------------------------------------------------------------------------
 proc tablelist::getCellFont {win key col} {
     upvar ::tablelist::ns${win}::data data
-
     if {[info exists data($key,$col-font)]} {
 	return $data($key,$col-font)
     } elseif {[info exists data($key-font)]} {
@@ -3154,16 +3296,18 @@ proc tablelist::getCellFont {win key col} {
 # still unknown.
 #------------------------------------------------------------------------------
 proc tablelist::reconfigWindows win {
-    upvar ::tablelist::ns${win}::data data
-
     #
     # Force any geometry manager calculations to be completed first
     #
     update idletasks
+    if {![winfo exists $win]} {			;# because of update idletasks
+	return ""
+    }
 
     #
     # Reconfigure the cells specified in the list data(cellsToReconfig)
     #
+    upvar ::tablelist::ns${win}::data data
     foreach cellIdx $data(cellsToReconfig) {
 	foreach {row col} [split $cellIdx ","] {}
 	set key [lindex [lindex $data(itemList) $row] end]
@@ -3183,7 +3327,6 @@ proc tablelist::reconfigWindows win {
 #------------------------------------------------------------------------------
 proc tablelist::isCellEditable {win row col} {
     upvar ::tablelist::ns${win}::data data
-
     set key [lindex [lindex $data(itemList) $row] end]
     if {[info exists data($key,$col-editable)]} {
 	return $data($key,$col-editable)
@@ -3200,7 +3343,6 @@ proc tablelist::isCellEditable {win row col} {
 #------------------------------------------------------------------------------
 proc tablelist::getEditWindow {win row col} {
     upvar ::tablelist::ns${win}::data data
-
     set key [lindex [lindex $data(itemList) $row] end]
     if {[info exists data($key,$col-editwindow)]} {
 	return $data($key,$col-editwindow)
